# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as BaseProvider

# Source: Gist
# https://raw.githubusercontent.com/neves/escolar/master/profissoes.txt


class Provider(BaseProvider):
    jobs = [
        "Acompanhante",
        "Açougueiro",
        "Acupunturista",
        "Adestrador de animais",
        "Administrador",
        "Administrador de banco de dados DBA",
        "Administrador de redes",
        "Administrador público",
        "Advogado",
        "Aeromoça",
        "Aeronauta",
        "Aeroviário",
        "Afiador de ferramentas",
        "Agente comunitário de saúde",
        "Agente de combate à endemias",
        "Agente de defesa sanitária",
        "Agente de defesa sanitária animal",
        "Agente de viagens",
        "Agente funerário",
        "Agente penitenciário",
        "Agricultor",
        "Agrimensor",
        "Agrônomo",
        "Ajudante de produção",
        "Alergologista",
        "Alfaiate",
        "Almirante",
        "Almoxarife",
        "Alpinista",
        "Ambientalista",
        "Ambulante",
        "Amolador de ferramentas",
        "Analista de sistemas",
        "Anestesiologista",
        "Angiologista",
        "Antropólogo",
        "Apicultor",
        "Apontador de mão-de-obra",
        "Apresentador",
        "Árbitro e mediador",
        "Argumentista",
        "Armador",
        "Armeiro",
        "Arqueólogo",
        "Arquiteto",
        "Arquivista",
        "Arranjador musical",
        "Arrumadeira",
        "Artesão",
        "Artista de circo",
        "Artista plástico",
        "Artista/Técnico em espetáculos de diversões",
        "Ascensorista",
        "Assessor de imprensa",
        "Assessor parlamentar",
        "Assistente administrativo",
        "Assistente de câmera",
        "Assistente de direção",
        "Assistente de produção",
        "Assistente social",
        "Astrofísico",
        "Astrólogo",
        "Astronauta",
        "Astrônomo",
        "Atendente",
        "Atleta de arremesso de peso",
        "Atleta de canoagem",
        "Atleta de nado sincronizado",
        "Atleta de tiro com arco",
        "Ator",
        "Atuário",
        "Auditor",
        "Auxiliar administrativo",
        "Auxiliar de reprografia",
        "Auxiliar de serviços gerais",
        "Avalista",
        "Aviador",
        "Babá",
        "Babysitter",
        "Bailarina",
        "Baixista",
        "Balconista",
        "Bancário",
        "Barbeiro",
        "Barman",
        "Bartender",
        "Baterista",
        "Bedel",
        "Berçarista",
        "Bibliotecário",
        "Biblioteconomista",
        "Biólogo",
        "Biomédico",
        "Bioquímico",
        "Biotecnólogo",
        "Bóia-fria",
        "Bombeiro",
        "Borracheiro",
        "Boticário",
        "Boxeador",
        "Brigadeiro",
        "Broker/Corretor da bolsa de valores",
        "Cabeleireiro",
        "Cabo",
        "Caça-talentos/Olheiro",
        "Cadeirinha",
        "Cadista",
        "Caixa",
        "Caldeireiro",
        "Cambista",
        "Camelô",
        "Cameraman",
        "Caminhoneiro",
        "Cancerologista ou Oncologista",
        "Cantor",
        "Capataz",
        "Capelão",
        "Capitão",
        "Capoeirista",
        "Cardiologista",
        "Carnavalesco",
        "Carpinteiro",
        "Cartazeiro",
        "Carteiro",
        "Cartógrafo",
        "Cartunista",
        "Catador de carangueijos",
        "Catador de material reciclável",
        "Cenógrafo",
        "Cenotécnico",
        "Ceramista",
        "Cerimonialista",
        "Chapeiro",
        "Chargista",
        "Chaveiro",
        "Chefe de cozinha",
        "Ciclista",
        "Cientista",
        "Cientista da informação e documentação",
        "Cientista de alimentos",
        "Cientista político",
        "Cientista social",
        "Cineasta",
        "Cinegrafista",
        "Cinematográfo",
        "Cirurgião bucal",
        "Cirurgião dentista",
        "Clap loader",
        "Clarinetista",
        "Classificador contábil",
        "Clínico geral",
        "Co-piloto",
        "Coach",
        "Cobaia Médica",
        "Cobrador de ônibus",
        "Cobrador de pedágio",
        "Coloproctologista",
        "Comandante",
        "Comerciante",
        "Comissário de bordo",
        "Compositor",
        "Comprador",
        "Confeiteiro",
        "Conferente de carga e descarga",
        "Conferente de expedição",
        "Conferente de recebimento",
        "Construtor",
        "Consultor",
        "Consultor de moda",
        "Consultor de radiestesia",
        "Contábil",
        "Contabilista",
        "Contador",
        "Contatólogo",
        "Continuista",
        "Contra regra",
        "Contramestre em transporte marítimo",
        "Controlador de vôo",
        "Controller",
        "Coordenador",
        "Copeiro",
        "Coreógrafo",
        "Coronel",
        "Corredor de atletismo",
        "Corregedor de justiça",
        "Corretor da bolsa de valores",
        "Corretor de imóveis",
        "Corretor de seguros",
        "Cortador de cana-de-açucar",
        "Costureira",
        "Coveiro/Sepultador",
        "Cozinheira",
        "Crítico",
        "Cumim",
        "Dançarino",
        "Datilógrafo",
        "Dedetizador",
        "Defensor Público",
        "Degustador",
        "Delegado",
        "Dentista",
        "Deputado",
        "Dermatologista",
        "Desembargador de justiça",
        "Desenhista",
        "Designer de interiores",
        "Designer de jóia",
        "Designer de moda",
        "Designer de produto ou desenhista industrial",
        "Designer gráfico",
        "Despachante",
        "Diagramador",
        "Dialoguista",
        "Diarista",
        "Digitador",
        "Diplomata",
        "Diretor de cinema",
        "Diretor de fotografia",
        "Diretor de produção",
        "DJ",
        "Dogueiro",
        "Dublador",
        "Dublê",
        "Ecólogo",
        "Economista",
        "Economista doméstico",
        "Editor",
        "Editor de mesa de corte",
        "Educador",
        "Educador integrado à saúde pública",
        "Eletricista",
        "Eletricista de automóveis",
        "Embaixador",
        "Embalador",
        "Embalsamador",
        "Empacotador",
        "Empregado doméstico",
        "Empresário",
        "Encanador",
        "Encarregado de manutenção predial",
        "Endocrinologista",
        "Endodontista",
        "Enfermeiro",
        "Engenheiro acústico",
        "Engenheiro aeronáutico",
        "Engenheiro agrícola",
        "Engenheiro agrimensor",
        "Engenheiro agrônomo",
        "Engenheiro ambiental",
        "Engenheiro cartográfico",
        "Engenheiro civil",
        "Engenheiro de alimentos",
        "Engenheiro de aquicultura",
        "Engenheiro de computação",
        "Engenheiro de controle e automação",
        "Engenheiro de energia",
        "Engenheiro de ergonomia",
        "Engenheiro de horticultura",
        "Engenheiro de iluminação",
        "Engenheiro de manufatura",
        "Engenheiro de materiais",
        "Engenheiro de minas",
        "Engenheiro de petróleo",
        "Engenheiro de processos",
        "Engenheiro de produção agroindustrial",
        "Engenheiro de produto ou produção",
        "Engenheiro de projetos",
        "Engenheiro de segurança do trabalho",
        "Engenheiro de som",
        "Engenheiro de supply chain ou logística",
        "Engenheiro de telecomunicações",
        "Engenheiro de transportes",
        "Engenheiro elétrico",
        "Engenheiro físico",
        "Engenheiro florestal",
        "Engenheiro industrial",
        "Engenheiro mecânico",
        "Engenheiro mecatrônico",
        "Engenheiro metalúrgico",
        "Engenheiro naval",
        "Engenheiro petroquímico",
        "Engenheiro químico",
        "Engenheiro sanitarista",
        "Engenheiro têxtil",
        "Engraxate",
        "Enólogo",
        "Entalhador",
        "Epidemiólogo",
        "Escoteiro",
        "Escritor",
        "Escriturário",
        "Escrivão",
        "Escultor",
        "Esgrimista",
        "Especialista em agronegócios",
        "Espeleologista",
        "Estampador de tecidos",
        "Estatístico",
        "Esteticista",
        "Estilista",
        "Estivador",
        "Estofador",
        "Estoquista",
        "Farmacêutico",
        "Faturista",
        "Faxineiro",
        "Feirante",
        "Ferramenteiro",
        "Ferreiro",
        "Ferroviário",
        "Figurante",
        "Figurinista",
        "Filósofo",
        "Fiscal",
        "Físico",
        "Físico nuclear",
        "Fisiculturista",
        "Fisioterapeuta",
        "Flanelinha",
        "Flautista",
        "Florista",
        "Fonoaudiólogo",
        "Forneiro",
        "Fotógrafo",
        "Frentista",
        "Fresador",
        "Fundidor",
        "Fundidor de placa de gesso",
        "Funileiro",
        "Gagsman",
        "Gandula",
        "Garçom",
        "Gari",
        "Garimpeiro",
        "Gastroenterologista",
        "Gastrônomo",
        "General",
        "Geofísico",
        "Geógrafo",
        "Geólogo",
        "Geradorista",
        "Gerente de banco",
        "Gerente de inovações ou novos negócios",
        "Gerente de riscos em seguros",
        "Gerente de vendas",
        "Geriatra",
        "Gestor ambiental",
        "Gestor de qualidade",
        "Gestor de recursos humanos",
        "Gestor de tecnologia da informação",
        "Gestor público",
        "Ginasta artística",
        "Ginasta rítmica",
        "Ginecologista",
        "Gourmet",
        "Governador",
        "Governanta",
        "Grafologista",
        "Gravurista",
        "Guarda ou policial rodoviário",
        "Guarda roupeiro",
        "Guardador de veículos",
        "Guia turistico",
        "Guincheiro",
        "Guitarrista",
        "Harpista",
        "Headhunter",
        "Hematologista",
        "Historiador",
        "Homeopata",
        "Hostess",
        "Ilustrador",
        "Implantodontista",
        "Impressor",
        "Imunologista",
        "Infectologista",
        "Inspetor",
        "Instalador de linha telefônica",
        "Instalador de painéis",
        "Instrumentador cirúrgico",
        "Instrumentista musical",
        "Instrutor",
        "Intérprete",
        "Intérprete de Bíblias",
        "Intérprete e tradutor de língua de sinais",
        "Investigador de Polícia",
        "Investigador particular",
        "Jangadeiro",
        "Jardineiro",
        "Jogador de badminton",
        "Jogador de basquete",
        "Jogador de bocha",
        "Jogador de boliche",
        "Jogador de futebol",
        "Jogador de golfe",
        "Jogador de handebol",
        "Jogador de hóquei",
        "Jogador de tênis de mesa",
        "Jogador de vôlei",
        "Jóquei",
        "Jornaleiro",
        "Jornalista",
        "Judoca",
        "Juiz de direito",
        "Juiz de futebol",
        "Juiz ou árbitro de futebol",
        "Karateca",
        "Kite-surfer",
        "Laboratorista",
        "Lactarista hospitalar",
        "Lamboteiro",
        "Lancheiro",
        "Lanterneiro",
        "Lapidário",
        "Lavador",
        "Lavador de veículos",
        "Leão de chácara",
        "Leiloeiro",
        "Leiteiro",
        "Lenhador",
        "Letrista",
        "Levantador de peso",
        "Líder comunitário",
        "Limpador de vidros",
        "Lixeiro/Coletor de lixo",
        "Locutor",
        "Lubrificador de máquinas",
        "Lutador de jiu-jitsu",
        "Lutador de karatê",
        "Lutador de kung fu",
        "Lutador de luta livre",
        "Lutador de taekwondo",
        "Luthier",
        "Mãe social",
        "Maestro",
        "Mágico",
        "Maitre",
        "Major",
        "Manicure",
        "Manobrista",
        "Maquiador",
        "Maquinista",
        "Marcador de luz",
        "Marceneiro",
        "Marechal",
        "Marinheiro",
        "Marketeiro",
        "Massagista",
        "Massoterapeuta",
        "Matemático",
        "Mecânico",
        "Mecânico de vôo",
        "Mecanógrafo",
        "Médico",
        "Médico cirurgião",
        "Médico geneticista",
        "Médico legista",
        "Médico nuclear",
        "Meeiro",
        "Mensageiro",
        "Meredeira",
        "Mergulhador",
        "Mestre cervejeiro",
        "Mestre-de-obras",
        "Metalúrgico",
        "Meteorologista",
        "Microfonista",
        "Militar da Aeronáutica",
        "Militar da Marinha",
        "Militar do Exército",
        "Ministro",
        "Modelista",
        "Modelo",
        "Moldador",
        "Moldureiro",
        "Moleiro",
        "Montador",
        "Montador de negativos",
        "Motofrete",
        "Motorista",
        "Mototaxista",
        "Museólogo",
        "Músico",
        "Musicoterapeuta",
        "Nadador",
        "Naturólogo",
        "Navegador",
        "Necromaquiador",
        "Nefrologista",
        "Neonatologista",
        "Neurocirurgião",
        "Neurologista",
        "Notário",
        "Numerólogo",
        "Nutricionista",
        "Nutrologista",
        "Obstetra",
        "Oceanógrafo",
        "Oculista",
        "Odontologista estético",
        "Odontologista legal",
        "Odontologista preventivo e social",
        "Odontopediatra",
        "Office-boy",
        "Oficial de justiça",
        "Oftalmologista",
        "Ombudsman",
        "Operador de bombas",
        "Operador de telemarketing",
        "Operador de vídeo",
        "Optometrista",
        "Orçamentista",
        "Orientador educacional",
        "Ortesista",
        "Ortodontista",
        "Ortopedista",
        "Ortoptista",
        "Otorrinolaringologista",
        "Ourives",
        "Paginador",
        "Paisagista",
        "Panfleteiro",
        "Panificador/Padeiro",
        "Paparazzo",
        "Papiloscopista",
        "Pára-quedista",
        "Paramédico",
        "Parteira tradicional",
        "Passador",
        "Pastilheiro",
        "Patinador",
        "Patologista",
        "Patologista oral",
        "Peão de rodeiro",
        "Pecuarista",
        "Pedagogo",
        "Pediatra",
        "Pedicure",
        "Pedreiro",
        "Peixeiro",
        "Penhorista",
        "Percursionista",
        "Perfumista",
        "Perfusionista",
        "Perito criminal",
        "Perito judicial",
        "Personal stylist",
        "Personal trainer",
        "Pescador",
        "Pesquisador",
        "Petroleiro",
        "Pianista",
        "Piloto automobilístico",
        "Piloto de avião",
        "Pintor",
        "Pizzaiolo",
        "Plastimodelista",
        "Pneumologista",
        "Podólogo",
        "Policial civil",
        "Policial federal",
        "Policial militar",
        "Polidor de produção",
        "Político",
        "Porteiro",
        "Portuário",
        "Prático",
        "Prefeito",
        "Prensista",
        "Preparador de máquinas",
        "Presidente da República",
        "Procurador de justiça",
        "Produtor de audio visual",
        "Produtor de eventos",
        "Produtor de multimídia",
        "Produtor editorial",
        "Produtor fonográfico",
        "Produtor musical",
        "Professor",
        "Profissional de áudio",
        "Profissional de cinema",
        "Profissional de comércio exterior",
        "Profissional de educação física",
        "Profissional de efeitos especiais",
        "Profissional de hotelaria",
        "Profissional de informática",
        "Profissional de linguística",
        "Profissional de logística",
        "Profissional de manutenção industrial",
        "Profissional de marketing",
        "Profissional de rádio e tv",
        "Profissional de reciclagem",
        "Profissional de recursos humanos",
        "Profissional de relacionamento com investidores RI",
        "Profissional de relações internacionais",
        "Profissional de relações públicas",
        "Profissional de tecnologia de laticínios",
        "Programador",
        "Projetista mecânico",
        "Promotor de eventos",
        "Promotor de vendas",
        "Promotor público/de justiça",
        "Protesista",
        "Protético dentário",
        "Psicólogo",
        "Psicomotricista",
        "Psicopedagogo",
        "Psiquiatra",
        "Publicitário",
        "Quadrinista",
        "Químico",
        "Químico farmacêutico",
        "Quiropraxista",
        "Quitandeiro",
        "Radialista",
        "Radialista programador",
        "Radiologista",
        "Radiooperador de vôo",
        "Radioterapêutico",
        "Rebarbador de metal",
        "Recepcionista",
        "Recreador",
        "Redator",
        "Regente",
        "Relações públicas",
        "Remador",
        "Repórter",
        "Repositor",
        "Representante comercial",
        "Restaurador",
        "Retificador",
        "Reumatologista",
        "Revendedor",
        "Revisor",
        "Roteirista",
        "Sacoleira",
        "Salgadeira",
        "Salva-vidas",
        "Sapateiro",
        "Sargento",
        "Saxofonista",
        "Secretária",
        "Seguidor de compras",
        "Segurança particular",
        "Selecionador de pessoal",
        "Senador",
        "Separador",
        "Seringueiro",
        "Serralheiro",
        "Servente-de-obras",
        "Serventuário",
        "Sexólogo",
        "Síndico",
        "Skatista",
        "Sociólogo",
        "Soldado",
        "Soldador",
        "Somelier",
        "Sonoplasta",
        "Subprefeito",
        "Supervisor",
        "Surfista",
        "Sushiman",
        "Tabelião",
        "Taifeiro",
        "Tapeceiro",
        "Tatuador",
        "Taxidermista/Embalsamador",
        "Taxista",
        "Tecelão",
        "Técnico de gesso",
        "Técnico de som",
        "Técnico em agropecuária",
        "Técnico em arquivo",
        "Técnico em aviação",
        "Técnico em desporto",
        "Técnico em documentação",
        "Técnico em edificações",
        "Técnico em hardware",
        "Técnico em higiene dentária",
        "Técnico em óptica",
        "Técnico em radiologia",
        "Técnico em rede",
        "Técnico em segurança do trabalho",
        "Técnico em taquigrafia",
        "Técnico em tratamento de água",
        "Técnico tributarista",
        "Tecnólogo em automação industrial",
        "Tecnólogo em Ciências das plantas medicinais",
        "Tecnólogo em desenvolvimento social",
        "Tecnólogo em esporte e lazer",
        "Tecnólogo em geoprocessamento",
        "Tecnólogo em irrigação e drenagem",
        "Tecnólogo em jogos digitais",
        "Tecnólogo em navegação fluvial",
        "Tecnólogo em negócios imobiliários",
        "Tecnólogo em papel e celulose",
        "Tecnólogo em processos químicos",
        "Tecnólogo em produção de bebidas",
        "Tecnólogo em produção moveleira",
        "Tecnólogo em produção Sucroalcooleira",
        "Tecnólogo em recursos pesqueiros",
        "Tecnólogo em rochas ornamentais",
        "Tecnólogo em silvicultura",
        "Tecnólogo em tecnologia da madeira",
        "Telefonista",
        "Telegrafista",
        "Tenente",
        "Tenista",
        "Teólogo",
        "Terapeuta floral",
        "Terapeuta Holístico",
        "Terapeuta ocupacional",
        "Tesoureiro",
        "Timoneiro",
        "Tintureiro",
        "Topógrafo",
        "Torneiro mecânico",
        "Torreiro/Torrista",
        "Tosador",
        "Toxicologista",
        "Tradutor",
        "Transcritor",
        "Transportador",
        "Traumatologista",
        "Treinador",
        "Triatleta",
        "Trilheiro ou músico de cinema",
        "Trompetista",
        "Turismólogo",
        "Ufólogo",
        "Urbanista",
        "Urologista",
        "Velejador",
        "Vendedor",
        "Ventríloquo",
        "Vereador",
        "Veterinário",
        "Vigia parlamentar",
        "Vigilante noturno/diurno",
        "Violonista",
        "Vistoriador de sinistros",
        "Viveirista",
        "Webdesigner",
        "Webmaster",
        "Windsurfer",
        "Xilógrafo",
        "Zelador",
        "Zootecnista",
    ]



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as SsnProvider
import datetime
import operator


def checksum(digits):
    """Calculate checksum of Estonian personal identity code.

    Checksum is calculated with "Modulo 11" method using level I or II scale:
    Level I scale: 1 2 3 4 5 6 7 8 9 1
    Level II scale: 3 4 5 6 7 8 9 1 2 3

    The digits of the personal code are multiplied by level I scale and summed;
    if remainder of modulo 11 of the sum is less than 10, checksum is the
    remainder.
    If remainder is 10, then level II scale is used; checksum is remainder if
    remainder < 10 or 0 if remainder is 10.

    See also https://et.wikipedia.org/wiki/Isikukood
    """
    sum_mod11 = sum(map(operator.mul, digits, Provider.scale1)) % 11
    if sum_mod11 < 10:
        return sum_mod11
    sum_mod11 = sum(map(operator.mul, digits, Provider.scale2)) % 11
    return 0 if sum_mod11 == 10 else sum_mod11


class Provider(SsnProvider):
    scale1 = (1, 2, 3, 4, 5, 6, 7, 8, 9, 1)
    scale2 = (3, 4, 5, 6, 7, 8, 9, 1, 2, 3)

    def ssn(self, min_age=16, max_age=90):
        """
        Returns 11 character Estonian personal identity code (isikukood, IK).

        Age of person is between 16 and 90 years, based on local computer date.
        This function assigns random sex to person.
        An Estonian Personal identification code consists of 11 digits,
        generally given without any whitespace or other delimiters.
        The form is GYYMMDDSSSC, where G shows sex and century of birth (odd
        number male, even number female, 1-2 19th century, 3-4 20th century,
        5-6 21st century), SSS is a serial number separating persons born on
        the same date and C a checksum.

        https://en.wikipedia.org/wiki/National_identification_number#Estonia
        """
        age = datetime.timedelta(
            days=self.generator.random.randrange(
                min_age * 365, max_age * 365))
        birthday = datetime.date.today() - age
        if birthday.year < 2000:
            ik = self.generator.random.choice(('3', '4'))
        elif birthday.year < 2100:
            ik = self.generator.random.choice(('5', '6'))
        else:
            ik = self.generator.random.choice(('7', '8'))

        ik += "%02d%02d%02d" % ((birthday.year % 100), birthday.month,
                                birthday.day)
        ik += str(self.generator.random.randrange(0, 999)).zfill(3)
        return ik + str(checksum([int(ch) for ch in ik]))



try:
    from inspect import getfullargspec as _getargspec
except ImportError:
    from inspect import getargspec as _getargspec


class getargspec(object):
    def __init__(self, method):
        self.argspec = _getargspec(method)

    @property
    def args(self):
        return self.argspec.args

    @property
    def varargs(self):
        return self.argspec.varargs

    @property
    def varkw(self):
        if hasattr(self.argspec, 'keywords'):
            return self.argspec.keywords
        return self.argspec.varkw

    @property
    def defaults(self):
        return self.argspec.defaults



# coding=utf-8
from __future__ import unicode_literals
from collections import OrderedDict
from .. import Provider as InternetProvider
from faker.utils.decorators import slugify


class Provider(InternetProvider):
    user_name_formats = (
        '{{last_romanized_name}}.{{first_romanized_name}}',
        '{{first_romanized_name}}.{{last_romanized_name}}',
        '{{first_romanized_name}}##',
        '?{{last_romanized_name}}',
    )

    tlds = OrderedDict((
        ('cn', 0.8),
        ('net', 0.1),
        ('com', 0.05),
        ('org', 0.05),
    ))

    second_level_domains = ('ac', 'com', 'edu', 'gov', 'mil', 'net', 'org',
                            'ah', 'bj', 'cq', 'fj', 'gd', 'gs', 'gz', 'gx',
                            'ha', 'hb', 'he', 'hi', 'hk', 'hl', 'hn', 'jl',
                            'js', 'jx', 'ln', 'mo', 'nm', 'nx', 'qh', 'sc',
                            'sd', 'sh', 'sn', 'sx', 'tj', 'xj', 'xz', 'yn', 'zj')

    domain_formats = (
        '##', '??',
        '{{first_romanized_name}}',
        '{{last_romanized_name}}',
        '{{first_romanized_name}}{{last_romanized_name}}',
        '{{last_romanized_name}}{{last_romanized_name}}',
        '{{first_romanized_name}}{{first_romanized_name}}',
    )

    @slugify
    def domain_word(self):
        pattern = self.random_element(self.domain_formats)
        if '#' in pattern or '?' in pattern:
            return self.bothify(pattern)
        else:
            return self.generator.parse(pattern)

    def domain_name(self, levels=1):
        if levels < 1:
            raise ValueError("levels must be greater than or equal to 1")
        if levels == 1:
            domain_word = self.domain_word()
            # Avoids he.cn as seen in issue #687
            while domain_word in self.second_level_domains:
                domain_word = self.domain_word()
            return domain_word + '.' + self.tld()
        elif levels == 2:
            my_tld = self.tld()
            if my_tld == 'cn':
                my_second_level = self.random_element(self.second_level_domains)
            else:
                my_second_level = self.domain_word()
            return self.domain_word() + '.' + my_second_level + '.' + my_tld
        else:
            return self.domain_word() + '.' + self.domain_name(levels - 1)



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as InternetProvider
from faker.utils.decorators import slugify


class Provider(InternetProvider):
    user_name_formats = (
        '{{last_romanized_name}}.{{first_romanized_name}}',
        '{{first_romanized_name}}.{{last_romanized_name}}',
        '{{first_romanized_name}}##',
        '?{{last_romanized_name}}',
    )
    tlds = ('com', 'com', 'com', 'net', 'org', 'tw', 'tw', 'tw')

    @slugify
    def domain_word(self):
        return self.generator.format('last_romanized_name')



# coding=utf-8

import re
import string

from faker.utils.distribution import choices_distribution, choices_distribution_unique


_re_hash = re.compile(r'#')
_re_perc = re.compile(r'%')
_re_excl = re.compile(r'!')
_re_at = re.compile(r'@')
_re_qm = re.compile(r'\?')
_re_cir = re.compile(r'\^')


class BaseProvider(object):

    __provider__ = 'base'
    __lang__ = None

    def __init__(self, generator):
        self.generator = generator

    def random_int(self, min=0, max=9999):
        """
        Returns a random integer between two values.

        :param min: lower bound value (inclusive; default=0)
        :param max: upper bound value (inclusive; default=9999)
        :returns: random integer between min and max
        """
        return self.generator.random.randint(min, max)

    def random_digit(self):
        """
        Returns a random digit/number
        between 0 and 9.
        """
        return self.generator.random.randint(0, 9)

    def random_digit_not_null(self):
        """
        Returns a random non-zero digit/number
        between 1 and 9.
        """
        return self.generator.random.randint(1, 9)

    def random_digit_or_empty(self):
        """
        Returns a random digit/number
        between 0 and 9 or an empty string.
        """
        if self.generator.random.randint(0, 1):
            return self.generator.random.randint(0, 9)
        else:
            return ''

    def random_digit_not_null_or_empty(self):
        """
        Returns a random non-zero digit/number
        between 1 and 9 or and empty string.
        """
        if self.generator.random.randint(0, 1):
            return self.generator.random.randint(1, 9)
        else:
            return ''

    def random_number(self, digits=None, fix_len=False):
        """
        Returns a random number with 1 digit (default, when digits==None),
        a random number with 0 to given number of digits, or a random number
        with given number to given number of digits (when ``fix_len==True``).

        :param digits: maximum number of digits
        :param fix_len:  should the number have fixed length?
        :returns: random number with 0 to given number of digits or
            fixed length number
        """
        if digits is None:
            digits = self.random_digit()
        if fix_len:
            return self.generator.random.randint(
                pow(10, digits - 1), pow(10, digits) - 1)
        else:
            return self.generator.random.randint(0, pow(10, digits) - 1)

    def random_letter(self):
        """Returns a random letter (between a-z and A-Z)."""
        return self.generator.random.choice(
            getattr(string, 'letters', string.ascii_letters))

    def random_letters(self, length=16):
        """Returns a random letter (between a-z and A-Z)."""
        return self.random_choices(
            getattr(string, 'letters', string.ascii_letters),
            length=length,
        )

    def random_lowercase_letter(self):
        """Returns a random lowercase letter (between a-z)."""
        return self.generator.random.choice(string.ascii_lowercase)

    def random_uppercase_letter(self):
        """Returns a random letter (between A-Z)."""
        return self.generator.random.choice(string.ascii_uppercase)

    def random_elements(self, elements=('a', 'b', 'c'), length=None, unique=False):
        fn = choices_distribution_unique if unique else choices_distribution

        if length is None:
            length = self.generator.random.randint(1, len(elements))

        if unique and length > len(elements):
            raise ValueError(
                "Sample length cannot be longer than the number of unique elements to pick from.")

        if isinstance(elements, dict):
            choices = elements.keys()
            probabilities = elements.values()
        else:
            if unique:
                # shortcut
                return self.generator.random.sample(elements, length)
            choices = elements
            probabilities = [1.0 for _ in range(len(choices))]

        return fn(
            list(choices),
            list(probabilities),
            self.generator.random,
            length=length,
        )

    def random_choices(self, elements=('a', 'b', 'c'), length=None):
        """
        Returns a list of random, non-unique elements from a passed object.

        If `elements` is a dictionary, the value will be used as
        a weighting element. For example::

            random_element({"{{variable_1}}": 0.5, "{{variable_2}}": 0.2, "{{variable_3}}": 0.2, "{{variable_4}}": 0.1})

        will have the following distribution:
            * `variable_1`: 50% probability
            * `variable_2`: 20% probability
            * `variable_3`: 20% probability
            * `variable_4`: 10% probability

        """
        return self.random_elements(elements, length, unique=False)

    def random_element(self, elements=('a', 'b', 'c')):
        """
        Returns a random element from a passed object.

        If `elements` is a dictionary, the value will be used as
        a weighting element. For example::

            random_element({"{{variable_1}}": 0.5, "{{variable_2}}": 0.2, "{{variable_3}}": 0.2, "{{variable_4}}": 0.1})

        will have the following distribution:
            * `variable_1`: 50% probability
            * `variable_2`: 20% probability
            * `variable_3`: 20% probability
            * `variable_4`: 10% probability

        """
        return self.random_elements(elements, length=1)[0]

    def random_sample(self, elements=('a', 'b', 'c'), length=None):
        """
        Returns a list of random unique elements for the specified length.
        Multiple occurrences of the same value increase its probability to be in the output.
        """
        return self.random_elements(elements, length, unique=True)

    def randomize_nb_elements(
            self,
            number=10,
            le=False,
            ge=False,
            min=None,
            max=None):
        """
        Returns a random value near number.

        :param number: value to which the result must be near
        :param le: result must be lower or equal to number
        :param ge: result must be greater or equal to number
        :returns: a random int near number
        """
        if le and ge:
            return number
        _min = 100 if ge else 60
        _max = 100 if le else 140
        nb = int(number * self.generator.random.randint(_min, _max) / 100)
        if min is not None and nb < min:
            nb = min
        if max is not None and nb > min:
            nb = max
        return nb

    def numerify(self, text='###'):
        """
        Replaces all placeholders in given text with randomized values,
        replacing: all hash sign ('#') occurrences with a random digit
        (from 0 to 9); all percentage sign ('%') occurrences with a
        random non-zero digit (from 1 to 9); all exclamation mark ('!')
        occurrences with a random digit (from 0 to 9) or an empty string;
        and all at symbol ('@') occurrences with a random non-zero digit
        (from 1 to 9) or an empty string.

        :param text: string to be parsed
        :returns: string with all numerical placeholders filled in
        """
        text = _re_hash.sub(
            lambda x: str(self.random_digit()),
            text)
        text = _re_perc.sub(
            lambda x: str(self.random_digit_not_null()),
            text)
        text = _re_excl.sub(
            lambda x: str(self.random_digit_or_empty()),
            text)
        text = _re_at.sub(
            lambda x: str(self.random_digit_not_null_or_empty()),
            text)
        return text

    def lexify(self, text='????', letters=string.ascii_letters):
        """
        Replaces all question mark ('?') occurrences with a random letter.

        :param text: string to be parsed
        :param letters: a set of letters to choose from.
        :returns: string with all letter placeholders filled in
        """
        return _re_qm.sub(lambda x: self.random_element(letters), text)

    def bothify(self, text='## ??', letters=string.ascii_letters):
        """
        Replaces all placeholders with random numbers and letters.

        :param text: string to be parsed
        :returns: string with all numerical and letter placeholders filled in
        """
        return self.lexify(self.numerify(text), letters=letters)

    def hexify(self, text='^^^^', upper=False):
        """
        Replaces all circumflex ('^') occurrences with a random
        hexadecimal character.

        :param text: string to be parsed
        :param upper: Format as uppercase hexadecimal
        :returns: string with all letter placeholders filled in
        """
        letters = string.hexdigits[:-6]
        if upper:
            letters = letters.upper()
        return _re_cir.sub(lambda x: self.random_element(letters), text)



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as SsnProvider
from functools import reduce
from math import fmod


def zfix(d):
    if d < 10:
        return "0" + str(d)
    else:
        return d


class Provider(SsnProvider):
    def ssn(self, dob=None, gender=None):
        """
        Generates Hungarian SSN equivalent (személyazonosító szám or, colloquially, személyi szám)

        :param dob: date of birth as a "YYMMDD" string - this determines the checksum regime and is also encoded
            in the személyazonosító szám.
        :type dob: str
        :param gender: gender of the person - "F" for female, M for male.
        :type gender: str
        :return: személyazonosító szám in str format (11 digs)
        :rtype: str
        """
        # Hungarian SSNs consist of 11 decimal characters, of the following
        # schema:
        #
        #       M EEHHNN SSSK
        #       ↑    ↑    ↑ ↑
        #  gender  bday ser check digit
        #
        #
        #  The M (gender) character
        #  ------------------------
        #
        #  Born <= 1999        Born > 1999
        #  Male  Female        Male Female
        #   1      2             3     4
        #
        #  It also includes information on original citizenship,but this is
        #  ignored for the sake of simplicity.
        #
        #  Birthday
        #  --------
        #
        #  Simply encoded as EEHHNN.
        #
        #
        #  Serial
        #  ------
        #
        #  These digits differentiate persons born on the same date.
        #
        #
        #  Check digit
        #  -----------
        #
        #  For those born before 1996:
        #
        #  k11 = (1k1 + 2k2 + 3k3... 10k10) mod 11
        #
        #  That is, you multiply each digit with its ordinal, add it up and
        #  take it mod 11. After 1996:
        #
        #  k11 = (10k1 + 9k2 + 8k3... 1k10) mod 11
        #

        if dob:
            E = int(dob[0:2])
            H = int(dob[2:4])
            N = int(dob[4:6])

            if E <= 17:
                # => person born after '99 in all likelihood...
                if gender:
                    if gender.upper() == "F":
                        M = 4
                    elif gender.upper() == "M":
                        M = 3
                    else:
                        raise ValueError("Unknown gender - specify M or F.")
                else:
                    M = self.generator.random_int(3, 4)
            else:
                # => person born before '99.
                if gender:
                    if gender.upper() == "F":
                        M = 2
                    elif gender.upper() == "M":
                        M = 1
                    else:
                        raise ValueError("Unknown gender - specify M or F.")
                else:
                    M = self.generator.random_int(1, 2)
        elif gender:
            # => assume statistically that the person will be born before '99.
            E = self.generator.random_int(17, 99)
            H = self.generator.random_int(1, 12)
            N = self.generator.random_int(1, 30)

            if gender.upper() == "F":
                M = 2
            elif gender.upper() == "M":
                M = 1
            else:
                raise ValueError("Unknown gender - specify M or F")
        else:
            M = self.generator.random_int(1, 2)
            E = self.generator.random_int(17, 99)
            H = self.generator.random_int(1, 12)
            N = self.generator.random_int(1, 30)

        H = zfix(H)
        N = zfix(N)
        S = "{}{}{}".format(self.generator.random_digit(
        ), self.generator.random_digit(), self.generator.random_digit())

        vdig = "{M}{E}{H}{N}{S}".format(M=M, E=E, H=H, N=N, S=S)

        if 17 < E < 97:
            cum = [(k + 1) * int(v) for k, v in enumerate(vdig)]
        else:
            cum = [(10 - k) * int(v) for k, v in enumerate(vdig)]

        K = fmod(reduce(lambda x, y: x + y, cum), 11)

        return vdig + str(int(K))



# coding=utf-8

from __future__ import unicode_literals
from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
        '{{first_name_male}} {{last_name}} {{suffix}}',
        '{{first_name_female}} {{last_name}} {{suffix}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}} {{suffix}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}} {{suffix}}',
    )

    # From
    # http://de.wiktionary.org/wiki/Verzeichnis:Deutsch/Liste_der_h%C3%A4ufigsten_m%C3%A4nnlichen_Vornamen_Deutschlands
    first_names_male = (
        'Abbas', 'Abdul', 'Abdullah', 'Abraham', 'Abram', 'Achim', 'Ada',
        'Adalbert', 'Adam', 'Adelbert', 'Adem', 'Adolf', 'Adrian', 'Ahmad',
        'Ahmed', 'Ahmet', 'Alan', 'Alban', 'Albert', 'Alberto', 'Albin',
        'Albrecht', 'Aldo', 'Aleksandar', 'Aleksander', 'Aleksandr', 'Aleksej',
        'Alessandro', 'Alex', 'Alexander', 'Alexandre', 'Alexandros', 'Alexei',
        'Alexej', 'Alf', 'Alfons', 'Alfonso', 'Alfred', 'Alfredo', 'Ali',
        'Alois', 'Aloys', 'Alwin', 'Amir', 'Anastasios', 'Anatol', 'Anatoli',
        'Anatolij', 'Andre', 'Andreas', 'Andree', 'Andrei', 'Andrej', 'Andres',
        'Andrew', 'Andrey', 'Andrzej', 'André', 'Andy', 'Angelo', 'Anselm',
        'Ansgar', 'Ante', 'Anthony', 'Anto', 'Anton', 'Antonino', 'Antonio',
        'Antonios', 'Antonius', 'Apostolos', 'Aribert', 'Arif', 'Armin', 'Arnd',
        'Arndt', 'Arne', 'Arnfried', 'Arnim', 'Arno', 'Arnold', 'Arnulf',
        'Arthur', 'Artur', 'Athanasios', 'Attila', 'August', 'Augustin', 'Axel',
        'Aziz', 'Baldur', 'Balthasar', 'Baptist', 'Bartholomäus', 'Bastian',
        'Bayram', 'Bekir', 'Bela', 'Ben', 'Benedikt', 'Benjamin', 'Benno',
        'Berend', 'Bernard', 'Bernd', 'Bernd-Dieter', 'Berndt', 'Bernfried',
        'Bernhard', 'Bernt', 'Bernward', 'Bert', 'Berthold', 'Bertold',
        'Bertram', 'Birger', 'Björn', 'Bodo', 'Bogdan', 'Boris', 'Branko',
        'Brian', 'Bruno', 'Burckhard', 'Burghard', 'Burkard', 'Burkhard',
        'Burkhardt', 'Calogero', 'Carl', 'Carl-Heinz', 'Carlo', 'Carlos',
        'Carmelo', 'Carsten', 'Celal', 'Cemal', 'Cemil', 'Cengiz', 'Cetin',
        'Charles', 'Christian', 'Christof', 'Christoph', 'Christopher',
        'Christos', 'Claas', 'Claudio', 'Claudius', 'Claus', 'Claus-Dieter',
        'Claus-Peter', 'Clemens', 'Conrad', 'Constantin', 'Cord', 'Cornelius',
        'Cosimo', 'Curt', 'Czeslaw', 'Dagobert', 'Damian', 'Dan', 'Daniel',
        'Daniele', 'Danilo', 'Danny', 'Darius', 'Dariusz', 'Darko', 'David',
        'Denis', 'Dennis', 'Denny', 'Detlef', 'Detlev', 'Diedrich', 'Dierk',
        'Dieter', 'Diethard', 'Diethelm', 'Diether', 'Dietmar', 'Dietrich',
        'Dimitri', 'Dimitrios', 'Dino', 'Dirk', 'Ditmar', 'Dittmar', 'Dogan',
        'Domenico', 'Dominic', 'Dominik', 'Donald', 'Dragan', 'Drago', 'Dursun',
        'Dusan', 'Eberhard', 'Eberhardt', 'Eckard', 'Eckart', 'Eckehard',
        'Eckhard', 'Eckhardt', 'Eckhart', 'Edelbert', 'Edgar', 'Edmund',
        'Eduard', 'Edward', 'Edwin', 'Egbert', 'Eggert', 'Egon', 'Ehrenfried',
        'Ehrhard', 'Eitel', 'Ekkehard', 'Ekkehart', 'Ekrem', 'Elias', 'Elmar',
        'Emanuel', 'Emil', 'Emin', 'Emmerich', 'Engelbert', 'Engin', 'Enno',
        'Enrico', 'Enver', 'Ercan', 'Erdal', 'Erdogan', 'Erhard', 'Erhardt',
        'Eric', 'Erich', 'Erik', 'Erkan', 'Ernest', 'Ernst', 'Ernst-August',
        'Ernst-Dieter', 'Ernst-Otto', 'Erol', 'Erwin', 'Eugen', 'Evangelos',
        'Ewald', 'Fabian', 'Falk', 'Falko', 'Faruk', 'Fatih', 'Fedor', 'Felix',
        'Ferdi', 'Ferdinand', 'Ferenc', 'Fernando', 'Filippo', 'Florian',
        'Folker', 'Folkert', 'Francesco', 'Francis', 'Francisco', 'Franco',
        'Franjo', 'Frank', 'Frank-Michael', 'Frank-Peter', 'Franz',
        'Franz Josef', 'Franz-Josef', 'Franz-Peter', 'Franz-Xaver', 'Fred', 'Freddy',
        'Frederic', 'Frederik', 'Fredi', 'Fredo', 'Fredy', 'Fridolin',
        'Friedbert', 'Friedemann', 'Frieder', 'Friedhelm', 'Friedhold',
        'Friedo', 'Friedrich', 'Friedrich-Karl', 'Friedrich-Wilhelm',
        'Frithjof', 'Fritz', 'Gabor', 'Gabriel', 'Gaetano', 'Gebhard', 'Geert',
        'Georg', 'George', 'Georgios', 'Gerald', 'Gerard', 'Gerd', 'Gereon',
        'Gerfried', 'Gerhard', 'Gerhardt', 'Gerhart', 'German', 'Gernot',
        'Gero', 'Gerold', 'Gerolf', 'Gert', 'Gerwin', 'Gilbert', 'Gino',
        'Giorgio', 'Giovanni', 'Gisbert', 'Giuseppe', 'Goran', 'Gordon',
        'Gottfried', 'Gotthard', 'Gotthilf', 'Gotthold', 'Gottlieb', 'Gottlob',
        'Gregor', 'Grzegorz', 'Guenter', 'Guenther', 'Guido', 'Guiseppe',
        'Gunar', 'Gundolf', 'Gunnar', 'Gunter', 'Gunther', 'Guntram', 'Gustav',
        'Götz', 'Günter', 'Günther', 'H.-Dieter', 'Hagen', 'Hajo', 'Hakan',
        'Halil', 'Hannes', 'Hanni', 'Hanno', 'Hanns', 'Hans', 'Hans D.',
        'Hans Dieter', 'Hans Georg', 'Hans Josef', 'Hans Jörg', 'Hans Jürgen',
        'Hans Peter', 'Hans-Adolf', 'Hans-Albert', 'Hans-Bernd', 'Hans-Christian',
        'Hans-Detlef', 'Hans-Dieter', 'Hans-Dietrich', 'Hans-Eberhard',
        'Hans-Erich', 'Hans-Friedrich', 'Hans-Georg', 'Hans-Gerd', 'Hans-Gerhard',
        'Hans-Günter', 'Hans-Günther', 'Hans-H.', 'Hans-Heinrich', 'Hans-Helmut',
        'Hans-Henning', 'Hans-Herbert', 'Hans-Hermann', 'Hans-Hinrich',
        'Hans-J.', 'Hans-Joachim', 'Hans-Jochen', 'Hans-Josef', 'Hans-Jörg',
        'Hans-Jürgen', 'Hans-Karl', 'Hans-Ludwig', 'Hans-Martin', 'Hans-Michael',
        'Hans-Otto', 'Hans-Peter', 'Hans-Rainer', 'Hans-Rudolf',
        'Hans-Theo', 'Hans-Ulrich', 'Hans-Uwe', 'Hans-Walter', 'Hans-Werner',
        'Hans-Wilhelm', 'Hans-Willi', 'Hans-Wolfgang', 'Hansgeorg',
        'Hansjoachim', 'Hansjörg', 'Hansjürgen', 'Hanspeter', 'Harald', 'Hardy',
        'Harm', 'Harold', 'Harri', 'Harro', 'Harry', 'Hartmut', 'Hartmuth',
        'Hartwig', 'Hasan', 'Hassan', 'Hasso', 'Heiko', 'Heimo', 'Heiner',
        'Heinfried', 'Heini', 'Heino', 'Heinrich', 'Heinz', 'Heinz Dieter',
        'Heinz-Dieter', 'Heinz-Georg', 'Heinz-Gerd', 'Heinz-Günter',
        'Heinz-Günther', 'Heinz-Joachim', 'Heinz-Josef', 'Heinz-Jürgen', 'Heinz-Otto',
        'Heinz-Peter', 'Heinz-Walter', 'Heinz-Werner', 'Heinz-Wilhelm', 'Heinz-Willi',
        'Helfried', 'Helge', 'Hellmut', 'Hellmuth', 'Helmar', 'Helmut',
        'Helmuth', 'Hendrik', 'Henner', 'Henning', 'Henrik', 'Henry', 'Henryk',
        'Herbert', 'Heribert', 'Hermann', 'Hermann Josef', 'Hermann-Josef',
        'Herrmann', 'Herwig', 'Hilmar', 'Hinrich', 'Holger', 'Holm', 'Horst',
        'Horst-Dieter', 'Horst-Günter', 'Horst-Peter', 'Hubert', 'Hubertus',
        'Hugo', 'Hüseyin', 'Ian', 'Ibrahim', 'Ignatz', 'Ignaz', 'Igor', 'Ilhan',
        'Ilias', 'Ilija', 'Ilja', 'Immo', 'Imre', 'Ingbert', 'Ingmar', 'Ingo',
        'Ingolf', 'Ioannis', 'Isidor', 'Ismail', 'Ismet', 'Istvan', 'Ivan',
        'Ivo', 'Iwan', 'Jacek', 'Jacob', 'Jakob', 'James', 'Jan', 'Jan-Peter',
        'Janko', 'Jann', 'Janos', 'Janus', 'Janusz', 'Jaroslav', 'Jaroslaw',
        'Jens', 'Jens-Peter', 'Jens-Uwe', 'Jerzy', 'Jiri', 'Joachim', 'Jobst',
        'Jochem', 'Jochen', 'Joerg', 'Johan', 'Johann', 'Johannes', 'John',
        'Jonas', 'Jonas', 'Jonathan', 'Jorge', 'Jose', 'Josef', 'Joseph',
        'Josip', 'Jost', 'José', 'Jovan', 'Jozef', 'Juan', 'Juergen', 'Julian',
        'Julius', 'Juri', 'Jurij', 'Justus', 'Jörg', 'Jörg-Peter', 'Jörgen',
        'Jörn', 'Jürgen', 'Kai-Uwe', 'Kamil', 'Karl', 'Karl Heinz', 'Karl-August',
        'Karl-Dieter', 'Karl-Ernst', 'Karl-Friedrich', 'Karl-Georg',
        'Karl-Hans', 'Karl-Heinrich', 'Karl-Heinz', 'Karl-Hermann', 'Karl-Josef',
        'Karl-Jürgen', 'Karl-Ludwig', 'Karl-Otto', 'Karl-Peter', 'Karl-Werner',
        'Karl-Wilhelm', 'Karlfried', 'Karlheinz', 'Karsten', 'Kasimir',
        'Kaspar', 'Kay-Uwe', 'Kazim', 'Kemal', 'Kenan', 'Kenneth', 'Kevin',
        'Kilian', 'Klaas', 'Klaus', 'Klaus Dieter', 'Klaus Peter', 'Klaus-D.',
        'Klaus-Dieter', 'Klaus-Günter', 'Klaus-Jürgen', 'Klaus-Michael',
        'Klaus-Peter', 'Klaus-Ulrich', 'Klaus-Werner', 'Klemens', 'Knud', 'Knut',
        'Konrad', 'Konstantin', 'Konstantinos', 'Korbinian', 'Kornelius',
        'Kristian', 'Krzysztof', 'Kunibert', 'Kuno', 'Kurt', 'Ladislaus',
        'Lambert', 'Lars', 'Laszlo', 'Laurenz', 'Leander', 'Leif', 'Leo',
        'Leon', 'Leonard', 'Leonardo', 'Leonhard', 'Leonid', 'Leopold',
        'Leszek', 'Linus', 'Lorenz', 'Lorenzo', 'Lothar', 'Louis', 'Luciano',
        'Ludger', 'Ludwig', 'Luigi', 'Luis', 'Lukas', 'Lutz', 'Magnus',
        'Mahmoud', 'Mahmut', 'Maik', 'Malte', 'Manfred', 'Manuel', 'Marc',
        'Marcel', 'Marco', 'Marcus', 'Marek', 'Marian', 'Marijan', 'Mario',
        'Marius', 'Mariusz', 'Mark', 'Marko', 'Markus', 'Marten', 'Martin',
        'Marvin', 'Massimo', 'Mathias', 'Mato', 'Matteo', 'Matthias',
        'Matthäus', 'Mattias', 'Maurice', 'Maurizio', 'Max', 'Maxim',
        'Maximilian', 'Mehdi', 'Mehmet', 'Meik', 'Meinhard', 'Meinolf',
        'Meinrad', 'Mesut', 'Metin', 'Micha', 'Michael', 'Michail', 'Michel',
        'Miguel', 'Mijo', 'Mike', 'Mikhail', 'Milan', 'Milos', 'Miodrag',
        'Mirco', 'Mirko', 'Miroslav', 'Miroslaw', 'Mohamed', 'Mohammad',
        'Mohammed', 'Moritz', 'Muharrem', 'Murat', 'Mustafa', 'Muzaffer',
        'Necati', 'Nick', 'Nico', 'Nicolai', 'Nicolas', 'Nicolaus', 'Niels',
        'Niklas', 'Niko', 'Nikola', 'Nikolai', 'Nikolaj', 'Nikolaos', 'Nikolas',
        'Nikolaus', 'Nils', 'Norbert', 'Norman', 'Nurettin', 'Nuri', 'Olaf',
        'Ole', 'Oliver', 'Orhan', 'Ortwin', 'Oscar', 'Oskar', 'Osman', 'Oswald',
        'Oswin', 'Otfried', 'Othmar', 'Otmar', 'Ottfried', 'Ottmar', 'Otto',
        'Ottokar', 'Ottomar', 'Paolo', 'Pascal', 'Pasquale', 'Patric',
        'Patrick', 'Patrik', 'Paul', 'Paul-Gerhard', 'Paul-Heinz', 'Paulo',
        'Pavel', 'Pawel', 'Pedro', 'Peer', 'Pero', 'Petar', 'Peter',
        'Peter-Michael', 'Petros', 'Philip', 'Philipp', 'Philippe', 'Phillip',
        'Pierre', 'Pietro', 'Piotr', 'Pirmin', 'Pius', 'Rafael', 'Raik',
        'Raimund', 'Rainer', 'Ralf', 'Ralf-Dieter', 'Ralf-Peter', 'Ralph',
        'Ramazan', 'Ramon', 'Randolf', 'Raphael', 'Raymond', 'Raymund', 'Recep',
        'Reginald', 'Reimar', 'Reimer', 'Reimund', 'Reinald', 'Reiner',
        'Reinhard', 'Reinhardt', 'Reinhart', 'Reinhold', 'Remo', 'Renato',
        'Rene', 'René', 'Reza', 'Ricardo', 'Richard', 'Rico', 'Rigo', 'Riza',
        'Robby', 'Robert', 'Roberto', 'Robin', 'Rocco', 'Rochus', 'Roderich',
        'Roger', 'Roland', 'Rolf', 'Rolf-Dieter', 'Rolf-Peter', 'Roman',
        'Romuald', 'Ron', 'Ronald', 'Ronny', 'Rouven', 'Roy', 'Ruben', 'Rudi',
        'Rudolf', 'Rudolph', 'Rupert', 'Ryszard', 'Rüdiger', 'Saban', 'Sabri',
        'Sahin', 'Salih', 'Salvatore', 'Sami', 'Samir', 'Samuel', 'Sandor',
        'Sandro', 'Sebastian', 'Sebastiano', 'Sedat', 'Selim', 'Senol', 'Sepp',
        'Serge', 'Sergei', 'Sergej', 'Sergio', 'Severin', 'Siegbert',
        'Siegfried', 'Sieghard', 'Siegmar', 'Siegmund', 'Siegward', 'Sigfried',
        'Sigismund', 'Sigmar', 'Sigmund', 'Sigurd', 'Silvester', 'Silvio',
        'Simon', 'Slavko', 'Slawomir', 'Slobodan', 'Stanislaus', 'Stanislav',
        'Stanislaw', 'Stavros', 'Stefan', 'Stefano', 'Steffen', 'Stephan',
        'Stephen', 'Steve', 'Steven', 'Stjepan', 'Sven', 'Swen', 'Sylvester',
        'Sylvio', 'Sönke', 'Sören', 'Sükrü', 'Süleyman', 'Tadeusz', 'Tassilo',
        'Thaddäus', 'Theo', 'Theobald', 'Theodor', 'Theodoros', 'Thies',
        'Thilo', 'Thomas', 'Thoralf', 'Thorben', 'Thorsten', 'Tibor', 'Till',
        'Tillmann', 'Tilman', 'Tilmann', 'Tilo', 'Tim', 'Timm', 'Timo', 'Tino',
        'Tobias', 'Tom', 'Tomas', 'Tomasz', 'Tomislav', 'Toni', 'Tony',
        'Toralf', 'Torben', 'Torsten', 'Traugott', 'Udo', 'Ulf', 'Uli',
        'Ullrich', 'Ulrich', 'Urban', 'Urs', 'Utz', 'Uwe', 'Vadim', 'Valentin',
        'Valerij', 'Vassilios', 'Veit', 'Veli', 'Victor', 'Viktor', 'Vincent',
        'Vincenzo', 'Vinko', 'Vinzenz', 'Vitali', 'Vito', 'Vittorio', 'Vitus',
        'Vladimir', 'Vlado', 'Volker', 'Volkhard', 'Volkmar', 'Waldemar',
        'Walfried', 'Walter', 'Walther', 'Wenzel', 'Werner', 'Wieland',
        'Wieslaw', 'Wigbert', 'Wilfried', 'Wilhelm', 'Willfried', 'Willi',
        'William', 'Willibald', 'Willibert', 'Willy', 'Winfried', 'Witold',
        'Wladimir', 'Wojciech', 'Woldemar', 'Wolf', 'Wolf-Dieter',
        'Wolf-Dietrich', 'Wolf-Rüdiger', 'Wolfgang', 'Wolfhard', 'Wolfram', 'Wulf',
        'Xaver', 'Yilmaz', 'Yusuf', 'Zbigniew', 'Zdravko', 'Zeki', 'Zeljko',
        'Zenon', 'Zlatko', 'Zoltan', 'Zoran',
    )

    # From
    # http://de.wiktionary.org/wiki/Verzeichnis:Deutsch/Liste_der_h%C3%A4ufigsten_weiblichen_Vornamen_Deutschlands
    first_names_female = (
        'Adele', 'Adelgunde', 'Adelheid', 'Adelinde', 'Adeline', 'Adina',
        'Adolfine', 'Adriana', 'Adriane', 'Aenne', 'Änne', 'Agata', 'Agatha',
        'Agathe', 'Agnes', 'Agnieszka', 'Albertine', 'Albina', 'Aleksandra',
        'Alena', 'Alexa', 'Alexandra', 'Alice', 'Alicia', 'Alicja', 'Alida',
        'Alina', 'Aline', 'Alla', 'Alma', 'Almut', 'Almuth', 'Aloisia',
        'Alwina', 'Alwine', 'Amalia', 'Amalie', 'Amanda', 'Amelie', 'Ana',
        'Anastasia', 'Andrea', 'Aneta', 'Anett', 'Anette', 'Angela', 'Angelica',
        'Angelika', 'Angelina', 'Angelique', 'Anica', 'Anika', 'Anita', 'Anja',
        'Anka', 'Anke', 'Ann', 'Ann-Kathrin', 'Anna', 'Anna-Lena', 'Anna-Luise',
        'Anna-Maria', 'Anna-Marie', 'Annaliese', 'Annamaria', 'Anne',
        'Anne-Kathrin', 'Anne-Katrin', 'Anne-Marie', 'Anne-Rose', 'Annedore',
        'Annegret', 'Annegrete', 'Annekatrin', 'Anneke', 'Annelene', 'Anneli',
        'Annelie', 'Annelies', 'Anneliese', 'Annelise', 'Annelore', 'Annemarie',
        'Annemie', 'Annerose', 'Annett', 'Annette', 'Anni', 'Annie', 'Annika',
        'Annita', 'Anny', 'Antje', 'Antoinette', 'Antonia', 'Antonie',
        'Antonietta', 'Antonina', 'Apollonia', 'Ariane', 'Arzu', 'Asta',
        'Astrid', 'Augusta', 'Auguste', 'Aurelia', 'Aynur', 'Ayse', 'Aysel',
        'Ayten', 'Babett', 'Babette', 'Barbara', 'Beata', 'Beate', 'Beatrice',
        'Beatrix', 'Belinda', 'Benita', 'Berit', 'Bernadette', 'Bernhardine',
        'Berta', 'Bertha', 'Betina', 'Betti', 'Bettina', 'Betty', 'Bianca',
        'Bianka', 'Birgid', 'Birgit', 'Birgitt', 'Birgitta', 'Birte', 'Birthe',
        'Blanka', 'Bozena', 'Branka', 'Brigitta', 'Brigitte', 'Brit', 'Brita',
        'Britt', 'Britta', 'Brunhild', 'Brunhilde', 'Bruni', 'Bärbel',
        'Camilla', 'Canan', 'Caren', 'Carin', 'Carina', 'Carla', 'Carmela',
        'Carmen', 'Carmine', 'Carola', 'Carolin', 'Carolina', 'Caroline',
        'Caterina', 'Catharina', 'Catherine', 'Cathleen', 'Cathrin', 'Catrin',
        'Cecilia', 'Centa', 'Chantal', 'Charlotte', 'Christa', 'Christa-Maria',
        'Christel', 'Christiana', 'Christiane', 'Christin', 'Christina',
        'Christine', 'Christl', 'Cilli', 'Cilly', 'Cindy', 'Claire', 'Clara',
        'Clarissa', 'Claudia', 'Cläre', 'Concetta', 'Conny', 'Constance',
        'Constanze', 'Cora', 'Cordula', 'Corina', 'Corinna', 'Corinne',
        'Cornelia', 'Cosima', 'Cristina', 'Cynthia', 'Cäcilia', 'Cäcilie',
        'Dagmar', 'Dajana', 'Damaris', 'Dana', 'Danica', 'Daniela', 'Danielle',
        'Danuta', 'Daria', 'Deborah', 'Delia', 'Denise', 'Desiree', 'Diana',
        'Diane', 'Dietlind', 'Dietlinde', 'Dina', 'Dolores', 'Donata', 'Dora',
        'Doreen', 'Dorina', 'Doris', 'Dorit', 'Dorle', 'Dorota', 'Dorothe',
        'Dorothea', 'Dorothee', 'Dragica', 'Dunja', 'Dörte', 'Dörthe', 'Edda',
        'Edelgard', 'Edeltraud', 'Edeltraut', 'Edeltrud', 'Edit', 'Edith',
        'Editha', 'Ehrentraud', 'Eileen', 'Ekaterina', 'Elena', 'Eleni',
        'Elenore', 'Eleonora', 'Eleonore', 'Elfi', 'Elfie', 'Elfriede', 'Elif',
        'Elisa', 'Elisabet', 'Elisabeth', 'Elise', 'Elizabeth', 'Elke', 'Ella',
        'Ellen', 'Elli', 'Ellinor', 'Elly', 'Elma', 'Elsa', 'Elsbeth', 'Else',
        'Elvira', 'Elwira', 'Elzbieta', 'Emilia', 'Emilie', 'Emine', 'Emma',
        'Emmi', 'Emmy', 'Erdmute', 'Erica', 'Erika', 'Erna', 'Ernestine',
        'Ester', 'Esther', 'Etta', 'Eugenia', 'Eugenie', 'Eva', 'Eva-Maria',
        'Eva-Marie', 'Evamaria', 'Evangelia', 'Evelin', 'Eveline', 'Evelyn',
        'Evelyne', 'Evi', 'Ewa', 'Fabienne', 'Fadime', 'Fanny', 'Fatima',
        'Fatma', 'Felicia', 'Felicitas', 'Felizitas', 'Filiz', 'Flora',
        'Florence', 'Florentine', 'Franca', 'Francesca', 'Francoise', 'Franka',
        'Franziska', 'Frauke', 'Frederike', 'Freia', 'Freya', 'Frida', 'Frieda',
        'Friedericke', 'Friederike', 'Friedhilde', 'Friedl', 'Friedlinde',
        'Gabi', 'Gabriela', 'Gabriele', 'Gabriella', 'Gaby', 'Galina',
        'Genoveva', 'Georgia', 'Georgine', 'Geraldine', 'Gerda', 'Gerdi',
        'Gerhild', 'Gerlind', 'Gerlinde', 'Gerta', 'Gerti', 'Gertraud',
        'Gertraude', 'Gertraut', 'Gertrud', 'Gertrude', 'Gesa', 'Gesche',
        'Gesine', 'Geza', 'Giesela', 'Gilda', 'Gina', 'Giovanna', 'Gisa',
        'Gisela', 'Gislinde', 'Gitta', 'Gitte', 'Giuseppina', 'Gloria',
        'Gordana', 'Grazyna', 'Greta', 'Gretchen', 'Grete', 'Gretel', 'Gretl',
        'Grit', 'Gudrun', 'Gudula', 'Gunda', 'Gundel', 'Gundi', 'Gundula',
        'Gunhild', 'Gusti', 'Gönül', 'Gülay', 'Gülsen', 'Gülten', 'Halina',
        'Hanife', 'Hanna', 'Hannah', 'Hannchen', 'Hanne', 'Hanne-Lore',
        'Hannelore', 'Hanny', 'Harriet', 'Hatice', 'Hedda', 'Hedi', 'Hedwig',
        'Hedy', 'Heide', 'Heide-Marie', 'Heidelinde', 'Heidelore', 'Heidemarie',
        'Heiderose', 'Heidi', 'Heidrun', 'Heike', 'Helen', 'Helena', 'Helene',
        'Helga', 'Hella', 'Helma', 'Helmtrud', 'Henni', 'Henny', 'Henri',
        'Henriette', 'Henrike', 'Herlinde', 'Herma', 'Hermine', 'Herta',
        'Hertha', 'Hilda', 'Hildburg', 'Hilde', 'Hildegard', 'Hildegart',
        'Hildegund', 'Hildegunde', 'Hilma', 'Hiltraud', 'Hiltrud', 'Hubertine',
        'Hulda', 'Hülya', 'Ida', 'Ildiko', 'Ilka', 'Ilona', 'Ilonka', 'Ilse',
        'Imelda', 'Imke', 'Ina', 'Ines', 'Inga', 'Inge', 'Ingeborg', 'Ingeburg',
        'Ingelore', 'Ingetraud', 'Ingetraut', 'Ingrid', 'Ingried', 'Inka',
        'Inken', 'Inna', 'Insa', 'Ira', 'Irena', 'Irene', 'Irina', 'Iris',
        'Irma', 'Irmela', 'Irmengard', 'Irmgard', 'Irmhild', 'Irmi',
        'Irmingard', 'Irmtraud', 'Irmtraut', 'Irmtrud', 'Isa', 'Isabel',
        'Isabell', 'Isabella', 'Isabelle', 'Isolde', 'Ivana', 'Ivanka',
        'Ivonne', 'Iwona', 'Jacqueline', 'Jadwiga', 'Jana', 'Jane', 'Janet',
        'Janett', 'Janette', 'Janin', 'Janina', 'Janine', 'Janna', 'Jaqueline',
        'Jasmin', 'Jasmina', 'Jeanette', 'Jeannette', 'Jeannine', 'Jelena',
        'Jennifer', 'Jenny', 'Jessica', 'Jessika', 'Jo', 'Joana', 'Joanna',
        'Johanna', 'Johanne', 'Jolanda', 'Jolanta', 'Jolanthe', 'Josefa',
        'Josefine', 'Josephine', 'Judith', 'Julia', 'Juliana', 'Juliane',
        'Julie', 'Justina', 'Justine', 'Jutta', 'Karen', 'Karin', 'Karina',
        'Karla', 'Karola', 'Karolin', 'Karolina', 'Karoline', 'Kata', 'Katalin',
        'Katarina', 'Katarzyna', 'Katerina', 'Katharina', 'Katharine',
        'Katherina', 'Kathi', 'Kathleen', 'Kathrin', 'Kathy', 'Kati', 'Katja',
        'Katrin', 'Katy', 'Kerstin', 'Kira', 'Kirsten', 'Kirstin', 'Klara',
        'Klaudia', 'Klothilde', 'Kläre', 'Konstanze', 'Kordula', 'Korinna',
        'Kornelia', 'Kreszentia', 'Kreszenz', 'Kriemhild', 'Krista',
        'Kristiane', 'Kristin', 'Kristina', 'Kristine', 'Krystyna', 'Kunigunda',
        'Kunigunde', 'Käte', 'Käthe', 'Käthi', 'Laila', 'Lara', 'Larissa',
        'Laura', 'Lea', 'Leila', 'Lena', 'Lene', 'Leni', 'Leokadia', 'Leonie',
        'Leonore', 'Leopoldine', 'Leyla', 'Lia', 'Liane', 'Lidia', 'Lidija',
        'Lidwina', 'Liesa', 'Liesbeth', 'Lieschen', 'Liesel', 'Lieselotte',
        'Lili', 'Lilian', 'Liliana', 'Liliane', 'Lilija', 'Lilli', 'Lilly',
        'Lilo', 'Lina', 'Linda', 'Lioba', 'Lisa', 'Lisbeth', 'Liselotte',
        'Lisette', 'Lissi', 'Lissy', 'Ljiljana', 'Ljubica', 'Ljudmila', 'Loni',
        'Lore', 'Loretta', 'Lotte', 'Lotti', 'Louise', 'Lucia', 'Lucie',
        'Ludmila', 'Ludmilla', 'Ludwina', 'Luisa', 'Luise', 'Luitgard', 'Luka',
        'Luzia', 'Luzie', 'Lydia', 'Madeleine', 'Madlen', 'Magarete', 'Magda',
        'Magdalena', 'Magdalene', 'Magret', 'Magrit', 'Maike', 'Maja',
        'Malgorzata', 'Mandy', 'Manja', 'Manuela', 'Mara', 'Marcella', 'Mareen',
        'Mareike', 'Mareile', 'Maren', 'Marga', 'Margaret', 'Margareta',
        'Margarete', 'Margaretha', 'Margarethe', 'Margarita', 'Margit',
        'Margita', 'Margitta', 'Margot', 'Margret', 'Margrit', 'Maria', 'Maria-Luise',
        'Maria-Theresia', 'Mariana', 'Marianna', 'Marianne', 'Marica',
        'Marie', 'Marie-Louise', 'Marie-Luise', 'Marie-Theres', 'Marie-Therese',
        'Mariechen', 'Mariele', 'Marieluise', 'Marietta', 'Marija', 'Marika',
        'Marina', 'Mariola', 'Marion', 'Marisa', 'Marit', 'Marita', 'Maritta',
        'Marjan', 'Marleen', 'Marlen', 'Marlene', 'Marlies', 'Marliese',
        'Marlis', 'Marta', 'Martha', 'Martina', 'Martine', 'Mary', 'Marzena',
        'Mathilde', 'Maya', 'Mechthild', 'Mechthilde', 'Mechtild', 'Meike',
        'Melanie', 'Melissa', 'Melita', 'Melitta', 'Meral', 'Mercedes',
        'Meryem', 'Meta', 'Mia', 'Michaela', 'Michaele', 'Michelle', 'Milena',
        'Milica', 'Milka', 'Mina', 'Minna', 'Mira', 'Mirella', 'Miriam',
        'Mirja', 'Mirjam', 'Mirjana', 'Miroslawa', 'Mona', 'Monica', 'Monika',
        'Monique', 'Monja', 'Myriam', 'Nada', 'Nadeschda', 'Nadeshda', 'Nadia',
        'Nadin', 'Nadine', 'Nadja', 'Nancy', 'Natali', 'Natalia', 'Natalie',
        'Natalija', 'Natalja', 'Natascha', 'Nathalie', 'Nelli', 'Nelly',
        'Nermin', 'Nevenka', 'Nicole', 'Nina', 'Nora', 'Norma', 'Notburga',
        'Nuran', 'Nuray', 'Nurten', 'Oda', 'Olav', 'Olena', 'Olga', 'Olivia',
        'Ortrud', 'Ortrun', 'Ottilie', 'Oxana', 'Pamela', 'Paola', 'Pascale',
        'Patricia', 'Patrizia', 'Paula', 'Paulina', 'Pauline', 'Peggy', 'Petra',
        'Philomena', 'Pia', 'Polina', 'Priska', 'Rabea', 'Radmila', 'Rahel',
        'Raisa', 'Raissa', 'Ramona', 'Raphaela', 'Rebecca', 'Rebekka', 'Regina',
        'Regine', 'Reingard', 'Reinhild', 'Reinhilde', 'Rena', 'Renata',
        'Renate', 'Reni', 'Resi', 'Ria', 'Ricarda', 'Rita', 'Romana', 'Romy',
        'Rosa', 'Rosa-Maria', 'Rosalia', 'Rosalie', 'Rosalinde', 'Rose', 'Rose-Marie',
        'Rosel', 'Roselinde', 'Rosemarie', 'Rosi', 'Rosina', 'Rosita',
        'Rosl', 'Rosmarie', 'Roswita', 'Roswitha', 'Rotraud', 'Rotraut', 'Ruth',
        'Ruthild', 'Sabina', 'Sabine', 'Sabrina', 'Samira', 'Sandra', 'Sandy',
        'Sara', 'Sarah', 'Sarina', 'Saskia', 'Selma', 'Semra', 'Senta',
        'Serpil', 'Sevim', 'Sibel', 'Sibilla', 'Sibille', 'Sibylla', 'Sibylle',
        'Sieglinde', 'Siegrid', 'Siegried', 'Siegrun', 'Siglinde', 'Sigrid',
        'Sigrun', 'Silja', 'Silke', 'Silva', 'Silvana', 'Silvia', 'Simona',
        'Simone', 'Sina', 'Sinaida', 'Slavica', 'Sofia', 'Sofie', 'Solveig',
        'Songül', 'Sonia', 'Sonja', 'Sophia', 'Sophie', 'Stefani', 'Stefania',
        'Stefanie', 'Steffi', 'Stella', 'Stephanie', 'Stilla', 'Susan',
        'Susana', 'Susann', 'Susanna', 'Susanne', 'Suse', 'Susi', 'Suzanne',
        'Svea', 'Svenja', 'Svetlana', 'Swantje', 'Swetlana', 'Sybilla',
        'Sybille', 'Sylke', 'Sylvana', 'Sylvia', 'Sylvie', 'Sylwia', 'Tabea',
        'Tamara', 'Tania', 'Tanja', 'Tatiana', 'Tatjana', 'Telse', 'Teresa',
        'Thea', 'Theda', 'Thekla', 'Theodora', 'Theres', 'Theresa', 'Therese',
        'Theresia', 'Tilly', 'Tina', 'Traude', 'Traudel', 'Traudl', 'Traute',
        'Trude', 'Trudel', 'Trudi', 'Tülay', 'Türkan', 'Ulla', 'Ulrike',
        'Undine', 'Ursel', 'Ursula', 'Urszula', 'Urte', 'Uschi', 'Uta', 'Ute',
        'Valentina', 'Valentine', 'Valeri', 'Valeria', 'Valerie', 'Valeska',
        'Vanessa', 'Vera', 'Verena', 'Veronica', 'Veronika', 'Veronique',
        'Vesna', 'Victoria', 'Viktoria', 'Viola', 'Violetta', 'Virginia',
        'Viviane', 'Walburga', 'Waldtraut', 'Walentina', 'Walli', 'Wally',
        'Waltraud', 'Waltraut', 'Waltrud', 'Wanda', 'Wencke', 'Wendelin',
        'Wenke', 'Wera', 'Wibke', 'Wiebke', 'Wilfriede', 'Wilhelmine', 'Wilma',
        'Wiltrud', 'Xenia', 'Yasemin', 'Yasmin', 'Yvette', 'Yvonne', 'Zdenka',
        'Zehra', 'Zenta', 'Zeynep', 'Zita', 'Zofia',
    )

    first_names = first_names_male + first_names_female

    last_names = (
        'Ackermann', 'Adler', 'Adolph', 'Albers', 'Anders', 'Atzler', 'Aumann',
        'Austermühle', 'Bachmann', 'Bähr', 'Bärer', 'Barkholz', 'Barth',
        'Bauer', 'Baum', 'Becker', 'Beckmann', 'Beer', 'Beier', 'Bender',
        'Benthin', 'Berger', 'Beyer', 'Bien', 'Biggen', 'Binner', 'Birnbaum',
        'Bloch', 'Blümel', 'Bohlander', 'Bonbach', 'Bolander', 'Bolnbach',
        'Bolzmann', 'Börner', 'Bohnbach', 'Boucsein', 'Briemer', 'Bruder',
        'Buchholz', 'Budig', 'Butte', 'Carsten', 'Caspar', 'Christoph',
        'Cichorius', 'Conradi', 'Davids', 'Dehmel', 'Dietz',
        'Dippel', 'Ditschlerin', 'Dobes', 'Döhn', 'Döring', 'Dörr', 'Dörschner',
        'Dowerg', 'Drewes', 'Drub', 'Drubin', 'Dussen van', 'Eberhardt',
        'Ebert', 'Eberth', 'Eckbauer', 'Ehlert', 'Eigenwillig', 'Eimer',
        'Ernst', 'Etzler', 'Etzold', 'Faust', 'Fechner', 'Fiebig', 'Finke',
        'Fischer', 'Flantz', 'Fliegner', 'Förster', 'Franke', 'Freudenberger',
        'Fritsch', 'Fröhlich', 'Gehringer', 'Geisel', 'Geisler', 'Geißler',
        'Gerlach', 'Gertz', 'Gierschner', 'Gieß', 'Girschner', 'Gnatz',
        'Gorlitz', 'Gotthard', 'Graf', 'Grein Groth', 'Gröttner', 'Gude',
        'Gunpf', 'Gumprich', 'Gute', 'Gutknecht', 'Haase', 'Haering', 'Hänel',
        'Häring', 'Hahn', 'Hamann', 'Hande', 'Harloff', 'Hartmann', 'Hartung',
        'Hauffer', 'Hecker', 'Heidrich', 'Hein', 'Heinrich', 'Heintze', 'Heinz',
        'Hellwig', 'Henck', 'Hendriks', 'Henk', 'Henschel', 'Hentschel',
        'Hering', 'Hermann', 'Herrmann', 'Hermighausen', 'Hertrampf', 'Heser',
        'Heß', 'Hesse', 'Hettner', 'Hethur', 'Heuser', 'Hiller', 'Heydrich',
        'Höfig', 'Hofmann', 'Holsten', 'Holt', 'Holzapfel', 'Hölzenbecher',
        'Hörle', 'Hövel', 'Hoffmann', 'Hornich', 'Hornig', 'Hübel', 'Huhn',
        'Jacob', 'Jacobi Jäckel', 'Jähn', 'Jäkel', 'Jäntsch', 'Jessel',
        'Jockel', 'Johann', 'Jopich', 'Junck', 'Juncken', 'Jungfer', 'Junitz',
        'Junk', 'Junken', 'Jüttner', 'Kabus', 'Kade', 'Käster', 'Kallert',
        'Kambs', 'Karge', 'Karz', 'Kaul', 'Kensy', 'Keudel', 'Killer',
        'Kitzmann', 'Klapp', 'Klemm', 'Klemt', 'Klingelhöfer', 'Klotz',
        'Knappe', 'Kobelt', 'Koch', 'Koch II', 'Köhler', 'Köster', 'Kohl',
        'Kostolzin', 'Kramer', 'Kranz', 'Krause', 'Kraushaar', 'Krebs', 'Krein',
        'Kreusel', 'Kroker', 'Kruschwitz', 'Kuhl', 'Kühnert', 'Kusch',
        'Lachmann', 'Ladeck', 'Lange', 'Langern', 'Lehmann', 'Liebelt',
        'Lindau', 'Lindner', 'Linke', 'Löchel', 'Löffler', 'Loos', 'Lorch',
        'Losekann', 'Löwer', 'Lübs', 'Mälzer', 'Mangold', 'Mans', 'Margraf',
        'Martin', 'Matthäi', 'Meister', 'Mende', 'Mentzel', 'Metz', 'Meyer',
        'Mielcarek', 'Mies', 'Misicher', 'Mitschke', 'Mohaupt', 'Mosemann',
        'Möchlichen', 'Mude', 'Mühle', 'Mülichen', 'Müller', 'Naser', 'Nerger',
        'Nette', 'Neureuther', 'Neuschäfer', 'Niemeier', 'Noack', 'Nohlmans',
        'Oderwald', 'Oestrovsky', 'Ortmann', 'Otto', 'Paffrath', 'Pärtzelt',
        'Patberg', 'Pechel', 'Pergande', 'Peukert', 'Pieper', 'Plath', 'Pohl',
        'Pölitz', 'Preiß', 'Pruschke', 'Putz', 'Rädel', 'Radisch', 'Reichmann',
        'Reinhardt', 'Reising', 'Renner', 'Reuter', 'Riehl', 'Ring', 'Ritter',
        'Rogge', 'Rogner', 'Rohleder', 'Röhrdanz', 'Röhricht', 'Roht', 'Römer',
        'Rörricht', 'Rose', 'Rosemann', 'Rosenow', 'Roskoth', 'Rudolph',
        'Ruppersberger', 'Ruppert', 'Rust', 'Sager', 'Salz', 'Säuberlich',
        'Sauer', 'Schaaf', 'Schacht', 'Schäfer', 'Scheel', 'Scheibe', 'Schenk',
        'Scheuermann', 'Schinke', 'Schleich', 'Schleich', 'auch Schlauchin',
        'Schlosser', 'Schmidt', 'Schmidtke', 'Schmiedecke', 'Schmiedt',
        'Schönland', 'Scholl', 'Scholtz', 'Scholz', 'Schomber', 'Schottin',
        'Schuchhardt', 'Schüler', 'Schulz', 'Schuster', 'Schweitzer',
        'Schwital', 'Segebahn', 'Seifert', 'Seidel', 'Seifert', 'Seip',
        'Siering', 'Söding', 'Sölzer', 'Sontag', 'Sorgatz', 'Speer', 'Spieß',
        'Stadelmann', 'Stahr', 'Staude', 'Steckel', 'Steinberg', 'Stey',
        'Stiebitz', 'Stiffel', 'Stoll', 'Stolze', 'Striebitz', 'Stroh',
        'Stumpf', 'Süßebier', 'Täsche', 'Textor', 'Thanel', 'Thies',
        'Tintzmann', 'Tlustek', 'Trapp', 'Trommler', 'Tröst', 'Trub', 'Trüb',
        'Trubin', 'Trupp', 'Tschentscher', 'Ullmann', 'Ullrich',
        'van der Dussen', 'Vogt', 'Vollbrecht', 'Wagenknecht', 'Wagner', 'Wähner',
        'Walter', 'Warmer', 'Weihmann', 'Weimer', 'Weinhage', 'Weinhold',
        'Weiß', 'Weitzel', 'Weller', 'Wende', 'Wernecke', 'Werner', 'Wesack',
        'Wiek', 'Wieloch', 'Wilms', 'Wilmsen', 'Winkler', 'Wirth', 'Wohlgemut',
        'Wulf', 'Wulff', 'Zahn', 'Zänker', 'Ziegert', 'Zimmer', 'Zirme',
        'Zobel', 'Zorbach',
    )

    prefixes_male = (
        'Herr', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )
    prefixes_female = (
        'Frau', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )

    suffixes = ('B.Sc.', 'B.A.', 'B.Eng.', 'MBA.')



# coding=utf-8
from __future__ import unicode_literals
from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{prefix}} {{first_name}} {{last_name}}',
        '{{first_name}} {{last_name}}',
        '{{prefix}} {{first_name}} {{last_name}}',
    )

    first_names = (
        'Alexander',
        'Alina',
        'Andreas',
        'Anna',
        'Anton',
        'Benjamin',
        'Bernhard',
        'Christian',
        'Christop',
        'Daniel',
        'David',
        'Dominik',
        'Elena',
        'Elias',
        'Emil',
        'Emilia',
        'Fabian',
        'Felix',
        'Florian',
        'Franz',
        'Fransizka',
        'Gabriel',
        'Gernot',
        'Hanna',
        'Ingrid',
        'Isabel'
        'Jakob',
        'Jana',
        'Jasmin',
        'Johanna',
        'Johannes',
        'Jonas',
        'Julia',
        'Julian',
        'Katharinna',
        'Konrad',
        'Konstantin'
        'Lara',
        'Laura',
        'Lena',
        'Leo',
        'Leon',
        'Linda',
        'Luca',
        'Lukas',
        'Marcel',
        'Maria',
        'Martin',
        'Matthias',
        'Max',
        'Maximilian',
        'Mia',
        'Michael',
        'Moritz',
        'Nico',
        'Niklas',
        'Nina',
        'Noah',
        'Oliver',
        'Olivia',
        'Paul',
        'Paula',
        'Philipp',
        'Pia',
        'Raphael',
        'Robert',
        'Samuel',
        'Sarah',
        'Sebastian',
        'Simon',
        'Sophie',
        'Theresa',
        'Thomas',
        'Tim',
        'Tobias',
        'Valentin')

    last_names = (
        'Auer', 'Aigner',
        'Bauer', 'Baumgartner', 'Berger', 'Binder', 'Brunner',
        'Cap', 'Capek', 'Cech', 'Chum',
        'Deng', 'Denk', 'Daume', 'Dienstl',
        'Ebner', 'Eder', 'Egger',
        'Fasching', 'Felber', 'Ferstel', 'Fichtner', 'Fischer', 'Fuchs',
        'Gasser', 'Gastegger', 'Geier', 'Geisler', 'Grabner', 'Gruber',
        'Haas', 'Haiden', 'Hofer', 'Holzer', 'Huber',
        'Illes', 'Ircher', 'Itzlinger',
        'Jahn', 'Jobst', 'Jung', 'Jungbauer', 'Just',
        'Kainz', 'Karl', 'Karner', 'Koller',
        'Lang', 'Lechner', 'Lehner', 'Leitner',
        'Maier', 'Mair', 'Maurer', 'Mayer', 'Mayr', 'Moser', 'Müllner',
        'Pichler', 'Pucher',
        'Reiter', 'Riegler',
        'Schmid', 'Schneider', 'Schuster', 'Schwarz', 'Stadler', 'Steiner',
        'Wallner', 'Weber', 'Weiss', 'Wieser', 'Wimmer', 'Winkler', 'Winter', 'Wolf',
    )

    prefixes = ('Dr.', 'Mag.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.')



# coding=utf-8
from __future__ import unicode_literals

from text_unidecode import unidecode

from .. import BaseProvider

from ipaddress import ip_address, ip_network, IPV4LENGTH, IPV6LENGTH

# from faker.generator import random
# from faker.providers.lorem.la import Provider as Lorem
from faker.utils.decorators import lowercase, slugify, slugify_unicode


localized = True


class _IPv4Constants:
    """
    IPv4 network constants used to group networks into different categories.
    Structure derived from `ipaddress._IPv4Constants`.

    Excluded network list is updated to comply with current IANA list of
    private and reserved networks.
    """
    _network_classes = {
        'a': ip_network('0.0.0.0/1'),
        'b': ip_network('128.0.0.0/2'),
        'c': ip_network('192.0.0.0/3'),
    }

    _linklocal_network = ip_network('169.254.0.0/16')

    _loopback_network = ip_network('127.0.0.0/8')

    _multicast_network = ip_network('224.0.0.0/4')

    # Three common private networks from class A, B and CIDR
    # to generate private addresses from.
    _private_networks = [
        ip_network('10.0.0.0/8'),
        ip_network('172.16.0.0/12'),
        ip_network('192.168.0.0/16'),
    ]

    # List of networks from which IP addresses will never be generated,
    # includes other private IANA and reserved networks from
    # ttps://www.iana.org/assignments/iana-ipv4-special-registry/iana-ipv4-special-registry.xhtml
    _excluded_networks = [
        ip_network('0.0.0.0/8'),
        ip_network('100.64.0.0/10'),
        ip_network('127.0.0.0/8'),
        ip_network('169.254.0.0/16'),
        ip_network('192.0.0.0/24'),
        ip_network('192.0.2.0/24'),
        ip_network('192.31.196.0/24'),
        ip_network('192.52.193.0/24'),
        ip_network('192.88.99.0/24'),
        ip_network('192.175.48.0/24'),
        ip_network('198.18.0.0/15'),
        ip_network('198.51.100.0/24'),
        ip_network('203.0.113.0/24'),
        ip_network('240.0.0.0/4'),
        ip_network('255.255.255.255/32'),
    ] + [
        _linklocal_network,
        _loopback_network,
        _multicast_network,
    ]


class Provider(BaseProvider):
    safe_email_tlds = ('org', 'com', 'net')
    free_email_domains = ('gmail.com', 'yahoo.com', 'hotmail.com')
    tlds = (
        'com', 'com', 'com', 'com', 'com', 'com', 'biz', 'info', 'net', 'org',
    )

    uri_pages = (
        'index', 'home', 'search', 'main', 'post', 'homepage', 'category',
        'register', 'login', 'faq', 'about', 'terms', 'privacy', 'author',
    )
    uri_paths = (
        'app', 'main', 'wp-content', 'search', 'category', 'tag', 'categories',
        'tags', 'blog', 'posts', 'list', 'explore',
    )
    uri_extensions = (
        '.html', '.html', '.html', '.htm', '.htm', '.php', '.php', '.jsp',
        '.asp',
    )

    user_name_formats = (
        '{{last_name}}.{{first_name}}',
        '{{first_name}}.{{last_name}}',
        '{{first_name}}##',
        '?{{last_name}}',
    )
    email_formats = (
        '{{user_name}}@{{domain_name}}',
        '{{user_name}}@{{free_email_domain}}',
    )
    url_formats = (
        'www.{{domain_name}}/',
        '{{domain_name}}/',
    )
    uri_formats = (
        '{{url}}',
        '{{url}}{{uri_page}}/',
        '{{url}}{{uri_page}}{{uri_extension}}',
        '{{url}}{{uri_path}}/{{uri_page}}/',
        '{{url}}{{uri_path}}/{{uri_page}}{{uri_extension}}',
    )
    image_placeholder_services = (
        'https://placeholdit.imgix.net/~text',
        '?txtsize=55&txt={width}x{height}&w={width}&h={height}',
        'https://www.lorempixel.com/{width}/{height}',
        'https://dummyimage.com/{width}x{height}',
        'https://placekitten.com/{width}/{height}',
        'https://placeimg.com/{width}/{height}/any',
    )

    replacements = ()

    def _to_ascii(self, string):
        for search, replace in self.replacements:
            string = string.replace(search, replace)

        string = unidecode(string)
        return string

    @lowercase
    def email(self, domain=None):
        if domain:
            email = '{0}@{1}'.format(self.user_name(), domain)
        else:
            pattern = self.random_element(self.email_formats)
            email = "".join(self.generator.parse(pattern).split(" "))
        return email

    @lowercase
    def safe_email(self):
        return '{}@example.{}'.format(
            self.user_name(), self.random_element(self.safe_email_tlds),
        )

    @lowercase
    def free_email(self):
        return self.user_name() + '@' + self.free_email_domain()

    @lowercase
    def company_email(self):
        return self.user_name() + '@' + self.domain_name()

    @lowercase
    def free_email_domain(self):
        return self.random_element(self.free_email_domains)

    @lowercase
    def ascii_email(self):
        pattern = self.random_element(self.email_formats)
        return self._to_ascii(
            "".join(self.generator.parse(pattern).split(" ")),
        )

    @lowercase
    def ascii_safe_email(self):
        return self._to_ascii(
            self.user_name() +
            '@example.' +
            self.random_element(self.safe_email_tlds),
        )

    @lowercase
    def ascii_free_email(self):
        return self._to_ascii(
            self.user_name() + '@' + self.free_email_domain(),
        )

    @lowercase
    def ascii_company_email(self):
        return self._to_ascii(
            self.user_name() + '@' + self.domain_name(),
        )

    @slugify_unicode
    def user_name(self):
        pattern = self.random_element(self.user_name_formats)
        username = self._to_ascii(
            self.bothify(self.generator.parse(pattern)).lower(),
        )
        return username

    @lowercase
    def domain_name(self, levels=1):
        """
        Produce an Internet domain name with the specified number of
        subdomain levels.

        >>> domain_name()
        nichols-phillips.com
        >>> domain_name(2)
        williamson-hopkins.jackson.com
        """
        if levels < 1:
            raise ValueError("levels must be greater than or equal to 1")
        if levels == 1:
            return self.domain_word() + '.' + self.tld()
        else:
            return self.domain_word() + '.' + self.domain_name(levels - 1)

    @lowercase
    @slugify_unicode
    def domain_word(self):
        company = self.generator.format('company')
        company_elements = company.split(' ')
        company = self._to_ascii(company_elements.pop(0))
        return company

    def tld(self):
        return self.random_element(self.tlds)

    def url(self, schemes=None):
        """
        :param schemes: a list of strings to use as schemes, one will chosen randomly.
        If None, it will generate http and https urls.
        Passing an empty list will result in schemeless url generation like "://domain.com".

        :returns: a random url string.
        """
        if schemes is None:
            schemes = ['http', 'https']

        pattern = '{}://{}'.format(
            self.random_element(schemes) if schemes else "",
            self.random_element(self.url_formats),
        )

        return self.generator.parse(pattern)

    def _random_ipv4_address_from_subnet(self, subnet, network=False):
        """
        Produces a random IPv4 address or network with a valid CIDR
        from within a given subnet.

        :param subnet: IPv4Network to choose from within
        :param network: Return a network address, and not an IP address
        """
        address = str(
            subnet[self.generator.random.randint(
                0, subnet.num_addresses - 1,
            )],
        )

        if network:
            address += '/' + str(self.generator.random.randint(
                subnet.prefixlen,
                subnet.max_prefixlen,
            ))
            address = str(ip_network(address, strict=False))

        return address

    def _exclude_ipv4_networks(self, networks, networks_to_exclude):
        """
        Exclude the list of networks from another list of networks
        and return a flat list of new networks.

        :param networks: List of IPv4 networks to exclude from
        :param networks_to_exclude: List of IPv4 networks to exclude
        :returns: Flat list of IPv4 networks
        """
        for network_to_exclude in networks_to_exclude:
            def _exclude_ipv4_network(network):
                """
                Exclude a single network from another single network
                and return a list of networks. Network to exclude
                comes from the outer scope.

                :param network: Network to exclude from
                :returns: Flat list of IPv4 networks after exclusion.
                          If exclude fails because networks do not
                          overlap, a single element list with the
                          orignal network is returned. If it overlaps,
                          even partially, the network is excluded.
                """
                try:
                    return list(network.address_exclude(network_to_exclude))
                except ValueError:
                    # If networks overlap partially, `address_exclude`
                    # will fail, but the network still must not be used
                    # in generation.
                    if network.overlaps(network_to_exclude):
                        return []
                    else:
                        return [network]

            networks = list(map(_exclude_ipv4_network, networks))

            # flatten list of lists
            networks = [
                item for nested in networks for item in nested
            ]

        return networks

    def ipv4_network_class(self):
        """
        Returns a IPv4 network class 'a', 'b' or 'c'.

        :returns: IPv4 network class
        """
        return self.random_element('abc')

    def ipv4(self, network=False, address_class=None, private=None):
        """
        Produce a random IPv4 address or network with a valid CIDR.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :param private: Public or private
        :returns: IPv4
        """
        if private is True:
            return self.ipv4_private(address_class=address_class,
                                     network=network)
        elif private is False:
            return self.ipv4_public(address_class=address_class,
                                    network=network)

        # if neither private nor public is required explicitly,
        # generate from whole requested address space
        if address_class:
            all_networks = [_IPv4Constants._network_classes[address_class]]
        else:
            # if no address class is choosen, use whole IPv4 pool
            all_networks = [ip_network('0.0.0.0/0')]

        # exclude special networks
        all_networks = self._exclude_ipv4_networks(
            all_networks,
            _IPv4Constants._excluded_networks,
        )

        # choose random network from the list
        random_network = self.generator.random.choice(all_networks)

        return self._random_ipv4_address_from_subnet(random_network, network)

    def ipv4_private(self, network=False, address_class=None):
        """
        Returns a private IPv4.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :returns: Private IPv4
        """
        # compute private networks from given class
        supernet = _IPv4Constants._network_classes[
            address_class or self.ipv4_network_class()
        ]

        private_networks = [
            subnet for subnet in _IPv4Constants._private_networks
            if subnet.overlaps(supernet)
        ]

        # exclude special networks
        private_networks = self._exclude_ipv4_networks(
            private_networks,
            _IPv4Constants._excluded_networks,
        )

        # choose random private network from the list
        private_network = self.generator.random.choice(private_networks)

        return self._random_ipv4_address_from_subnet(private_network, network)

    def ipv4_public(self, network=False, address_class=None):
        """
        Returns a public IPv4 excluding private blocks.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :returns: Public IPv4
        """
        # compute public networks
        public_networks = [_IPv4Constants._network_classes[
            address_class or self.ipv4_network_class()
        ]]

        # exclude private and excluded special networks
        public_networks = self._exclude_ipv4_networks(
            public_networks,
            _IPv4Constants._private_networks +
            _IPv4Constants._excluded_networks,
        )

        # choose random public network from the list
        public_network = self.generator.random.choice(public_networks)

        return self._random_ipv4_address_from_subnet(public_network, network)

    def ipv6(self, network=False):
        """Produce a random IPv6 address or network with a valid CIDR"""
        address = str(ip_address(self.generator.random.randint(
            2 ** IPV4LENGTH, (2 ** IPV6LENGTH) - 1)))
        if network:
            address += '/' + str(self.generator.random.randint(0, IPV6LENGTH))
            address = str(ip_network(address, strict=False))
        return address

    def mac_address(self):
        mac = [self.generator.random.randint(0x00, 0xff) for _ in range(0, 6)]
        return ":".join(map(lambda x: "%02x" % x, mac))

    def uri_page(self):
        return self.random_element(self.uri_pages)

    def uri_path(self, deep=None):
        deep = deep if deep else self.generator.random.randint(1, 3)
        return "/".join(
            self.random_elements(self.uri_paths, length=deep),
        )

    def uri_extension(self):
        return self.random_element(self.uri_extensions)

    def uri(self):
        pattern = self.random_element(self.uri_formats)
        return self.generator.parse(pattern)

    @slugify
    def slug(self, value=None):
        """Django algorithm"""
        if value is None:
            value = self.generator.text(20)
        return value

    def image_url(self, width=None, height=None):
        """
        Returns URL to placeholder image
        Example: http://placehold.it/640x480
        """
        width_ = width or self.random_int(max=1024)
        height_ = height or self.random_int(max=1024)
        placeholder_url = self.random_element(self.image_placeholder_services)
        return placeholder_url.format(width=width_, height=height_)
